@@ -2,10 +2,12 @@
     Credentials managing module.
 """
 import re
+import csv
 import json
 from pathlib import Path
 from functools import wraps
 
+import hcl2
 import click
 from click.exceptions import Exit
 import questionary
@@ -14,12 +16,16 @@ from ruamel.yaml import YAML
 
 from leverage import logger
 from leverage.path import get_home_path
-from leverage._internals import pass_state
+from leverage.path import get_global_config_path
+from leverage.path import NotARepositoryError
 from leverage.modules.terraform import awscli
 from leverage.modules.terraform import run as tfrun
 from leverage.modules.project import render_file
 from leverage.modules.project import PROJECT_CONFIG
+from leverage.modules.project import TEMPLATES_REPO_DIR
 from leverage.modules.project import load_project_config
+from leverage.conf import ENV_CONFIG_FILE
+from leverage.conf import load as load_env_config
 
 
 # Regexes for general validation
@@ -32,9 +38,13 @@ REGION = (r"[a-z]{2}-[gov-]?"
           r"(?:central|north|south|east|west|northeast|northwest|southeast|southwest|secret|topsecret)-[1-3]")
 ACCOUNT_ID = r"[0-9]{12}"
 MFA_SERIAL = fr"arn:aws:iam::{ACCOUNT_ID}:mfa/{USERNAME}"
-CREDENTIALS_FILE = fr"Access key ID,Secret access key\s+(?P<key_id>{KEY_ID}),(?P<secret_key>{SECRET_KEY})"
 
-AWSCLI_CONFIG_DIR = Path(get_home_path()) / ".aws"
+# TODO: Remove these and get them into the global app state
+try:
+    AWSCLI_CONFIG_DIR = Path(get_home_path()) / ".aws"
+    PROJECT_COMMON_TFVARS = Path(get_global_config_path()) / "common.tfvars"
+except NotARepositoryError:
+    AWSCLI_CONFIG_DIR = PROJECT_COMMON_TFVARS = ""
 
 PROFILES = {
     "bootstrap": {
@@ -89,7 +99,7 @@ def _ask_for_short_name():
         str: Project short name.
     """
     return questionary.text(
-        message="Short name:",
+        message="Project short name:",
         qmark=">",
         validate=lambda value: bool(re.fullmatch(PROJECT_SHORT, value)) or "The project short name should be a two letter lowercase word"
     ).ask()
@@ -103,29 +113,16 @@ def _ask_for_region():
         str: Region.
     """
     return questionary.text(
-        message="Region:",
+        message="Credentials default region:",
         qmark=">",
+        default="us-east-1",
         validate=lambda value: bool(re.fullmatch(REGION, value)) or "Invalid region."
     ).ask()
 
 
-@_exit_if_user_cancels_input
-def _ask_for_profile():
-    """ Prompt for profile selection.
-
-    Returns:
-        str : Profile to configure.
-    """
-    return questionary.select(
-        message="Select the credentials to set:",
-        qmark=">",
-        choices=[Choice(profile["choice_title"], value=name) for name, profile in PROFILES.items()]
-    ).ask()
-
-
 @_exit_if_user_cancels_input
 def _ask_for_credentials_overwrite(profile, skip_option_title, overwrite_option_title):
-    """ Prompt user with options regarding already existing credentials, whether to 
+    """ Prompt user with options regarding already existing credentials, whether to
     skip their configuration or overwrite them.
 
     Args:
@@ -176,14 +173,14 @@ def _ask_for_credentials_location():
             "message": "Path to access keys file:",
             "qmark": ">",
             "when": lambda qs: qs["input_type"] == "path",
-            "validate": lambda value: (Path(value).is_file() and Path(value).exists()) or "Path must be an existing file"
+            "validate": lambda value: (Path(value).expanduser().is_file() and Path(value).expanduser().exists()) or "Path must be an existing file"
         }
     ])
     if not location:
         return
 
     input_type = location.get("input_type")
-    return Path(location.get("path")) if input_type == "path" else input_type
+    return Path(location.get("path")).expanduser() if input_type == "path" else input_type
 
 
 @_exit_if_user_cancels_input
@@ -216,15 +213,67 @@ def _ask_for_credentials():
 
 
 @click.group()
-@pass_state
-def credentials(state):
+def credentials():
     """ Manage AWS cli credentials. """
+
+
+def _load_configs_for_credentials():
+    """ Load all required values to configure credentials.
+
+    Raises:
+        Exit: If no project has been already initialized in the system.
+
+    Returns:
+        dict: Values needed to configure a credential and update the files accordingly.
+    """
     logger.info("Loading configuration file.")
-    state.project_config = load_project_config()
+    project_config = load_project_config()
 
-    if not render_file("build.env"):
-        raise Exit(1)
+    logger.info("Loading project environment configuration file.")
+    env_config = load_env_config()
 
+    terraform_config = {}
+    if PROJECT_COMMON_TFVARS.exists():
+        logger.info("Loading Terraform common configuration.")
+        terraform_config = hcl2.loads(PROJECT_COMMON_TFVARS.read_text())
+
+    config_values = {}
+    config_values["short_name"] = (project_config.get("short_name")
+                                       or env_config.get("PROJECT")
+                                       or terraform_config.get("project")
+                                       or _ask_for_short_name())
+    config_values["project_name"] = (project_config.get("project_name")
+                                         or terraform_config.get("project_long"))
+
+    config_values["primary_region"] = (project_config.get("primary_region")
+                                           or terraform_config.get("region_primary")
+                                           or _ask_for_region())
+    config_values["secondary_region"] = terraform_config.get("region_secondary")
+
+    config_values["organization"] = {"accounts": []}
+    # Accounts defined in Terraform code take priority
+    terraform_accounts = terraform_config.get("accounts", {})
+    if terraform_accounts:
+        config_values["organization"]["accounts"].extend([
+            {
+                "name": account_name,
+                "email": account_info.get("email"),
+                "id": account_info.get("id")
+            }
+            for account_name, account_info in terraform_accounts.items()
+        ])
+    # Add accounts not found in terraform code
+    project_accounts = [
+        account for account in project_config.get("organization", {}).get("accounts", [])
+        if account.get("name") not in terraform_accounts
+    ]
+    if project_accounts:
+        config_values["organization"]["accounts"].extend(project_accounts)
+
+    config_values["mfa_enabled"] = env_config.get("MFA_ENABLED", "false")
+    config_values["terraform_image_tag"] = env_config.get("TERRAFORM_IMAGE_TAG", "1.0.9")
+
+    return config_values
 
 
 def _profile_is_configured(profile):
@@ -256,29 +305,27 @@ def _extract_credentials(file):
     Returns:
         str, str: Key ID, Secret Key
     """
-    match = re.match(pattern=fr"Access key ID,Secret access key\s+(?P<key_id>{KEY_ID}),(?P<secret_key>{SECRET_KEY})",
-                     string=file.read_text())
+    with open(file) as access_keys_file:
+        try:
+            keys = next(csv.DictReader(access_keys_file))
 
-    if not match:
-        click.echo("\nMalformed access keys file\n")
+        except csv.Error:
+            click.echo("\nMalformed access keys file\n")
+            raise Exit(1)
+
+    try:
+        access_key_id = keys["Access key ID"]
+        secret_access_key = keys["Secret access key"]
+
+    except KeyError:
+        click.echo("\nFields for keys not found in access keys file\n")
         raise Exit(1)
 
-    return match.groups()
+    if not re.match(KEY_ID, access_key_id) or not re.match(SECRET_KEY, secret_access_key):
+        click.echo("\nMalformed keys in access keys file\n")
+        raise Exit(1)
 
-
-def configure_default_profile(region):
-    """ Set default profile values.
-
-    Args:
-        region (str): Region.
-    """
-    default = {
-        "output": "json",
-        "region": region
-    }
-
-    for key, value in default.items():
-        awscli(f"configure set {key} {value}")
+    return access_key_id, secret_access_key
 
 
 def _backup_file(filename):
@@ -363,95 +410,21 @@ def _get_management_account_id(profile):
     return caller_identity["Account"]
 
 
-@credentials.command()
-@click.option("--file",
-              type=click.Path(exists=True, path_type=Path),
-              help="Path to AWS cli credentials file.")
-@click.option("--force",
-              is_flag=True,
-              help="Force credentials creation, even if they are already configured.")
-@pass_state
-def create(state, file, force):
-    """ Initialize credentials for the project.
-
-    Configure the required credentials for the bootstrap process and a default profile.
-    Fetch management account id and update project configuration file.
-    """
-    project_config = state.project_config
-
-    short_name = project_config.get("short_name") or _ask_for_short_name()
-    region = project_config.get("primary_region") or _ask_for_region()
-    profile_name = f"{short_name}-bootstrap"
-
-    credentials_dir = AWSCLI_CONFIG_DIR / short_name
-
-    if _profile_is_configured(profile=profile_name):
-        if not (force
-                    or _ask_for_credentials_overwrite(
-                        profile="bootstrap",
-                        skip_option_title="Skip credentials configuration.",
-                        overwrite_option_title="Overwrite current credentials."
-                    )
-                ):
-            logger.info("Exiting credentials configuration.")
-            return
-
-    logger.info("Configuring default profile information.")
-    configure_default_profile(region)
-    profiles_config = credentials_dir / "config"
-    logger.info(f"[bold]Default profile configured in:[/bold] {profiles_config.as_posix()}")
-
-    logger.info("Configuring [bold]bootstrap[/bold] credentials.")
-    configure_credentials(profile=profile_name, file=file)
-    credentials_config = credentials_dir / "credentials"
-    logger.info(f"[bold]Bootstrap credentials configured in:[/bold] {credentials_config.as_posix()}")
-
-    if not _credentials_are_valid(profile=profile_name):
-        logger.error("Invalid bootstrap credentials. Please check the given keys.")
-        return
-
-    accounts = project_config.get("organization").get("accounts")
-    management_account = next((account for account in accounts if account["name"] == "management"), None)
-    if management_account:
-        logger.info("Fetching management account id.")
-        management_account["id"] = _get_management_account_id(profile=profile_name)
-
-        logger.info("Updating project configuration file.")
-        YAML().dump(data=project_config, stream=PROJECT_CONFIG)
-
-    logger.info("Finished setting up [bold]bootstrap[/bold] credentials.")
-
-
-def _organization_is_created(profile):
-    """ Check if account is part of an organization.
-    Output when negative:
-        An error occurred (AWSOrganizationsNotInUseException) when calling the DescribeOrganization
-        operation: Your account is not a member of an organization.
-    Exit code when negative:
-        255
-
-    Args:
-        profile (str): Name of profile to configure.
-
-    Returns:
-        bool: Whether the organization exists or not.
-    """
-    exit_code, _ = awscli(f"--output json organizations describe-organization --profile {profile}")
-
-    return not exit_code
-
-
 def _get_organization_accounts(profile, project_name):
     """ Get organization accounts names and ids. Removing the prefixed project name from the account names.
 
     Args:
-        profile (str): Name of profile to configure.
-        project_name (str): Name of the project.
+        profile (str): Credentials profile.
+        project_name (str): Full name of the project.
 
     Returns:
         dict: Mapping of organization accounts names to ids.
     """
-    _, organization_accounts = awscli(f"--output json organizations list-accounts --profile {profile}")
+    exit_code, organization_accounts = awscli(f"--output json organizations list-accounts --profile {profile}")
+
+    if exit_code:
+        return {}
+
     organization_accounts = json.loads(organization_accounts)["Accounts"]
 
     prefix = f"{project_name}-"
@@ -494,21 +467,21 @@ def configure_profile(profile, values):
         awscli(f"configure set {key} {value} --profile {profile}")
 
 
-def configure_accounts_profiles(profile_name, region, organization_accounts, project_accounts):
+def configure_accounts_profiles(profile, region, organization_accounts, project_accounts):
     """ Set up the required profiles for all accounts to be used with AWS cli. Backup previous profiles.
 
     Args:
-        profile_name (str): Name of the profile to configure.
+        profile(str): Name of the profile to configure.
         region (str): Region.
         organization_accounts (dict): Name and id of all accounts in the organization.
         project_accounts (dict): Name and email of all accounts in project configuration file.
     """
-    short_name, profile = profile_name.split("-")
+    short_name, type = profile.split("-")
 
     mfa_serial = None
-    if PROFILES[profile]["mfa"]:
+    if PROFILES[type]["mfa"]:
         logger.info("Fetching MFA device serial.")
-        mfa_serial = _get_mfa_serial(profile_name)
+        mfa_serial = _get_mfa_serial(profile)
         if not mfa_serial:
             logger.error("No MFA device found for user. Please set up a device before configuring the accounts profiles.")
             raise Exit(1)
@@ -516,22 +489,26 @@ def configure_accounts_profiles(profile_name, region, organization_accounts, pro
     account_profiles = {}
     for account in project_accounts:
         account_name = account["name"]
-        try: # Account in config file may not be already created
-            account_id = organization_accounts[account_name]
-        except KeyError:
+        # DevOps roles do not have permission over management account
+        if "security" in profile and account_name == "management":
+            continue
+
+        # TODO: Add remaining profiles for remaining accounts declared in code if enough information is available
+        account_id = organization_accounts.get(account_name, account.get("id"))
+        if account_id is None:
             continue
 
         account_profile = {
             "output": "json",
             "region": region,
-            "role_arn": f"arn:aws:iam::{account_id}:role/{PROFILES[profile]['role']}",
-            "source_profile": profile_name
+            "role_arn": f"arn:aws:iam::{account_id}:role/{PROFILES[type]['role']}",
+            "source_profile": profile
         }
         if mfa_serial:
             account_profile["mfa_serial"] = mfa_serial
 
         # A profile identifier looks like `le-security-oaar`
-        account_profiles[f"{short_name}-{account_name}-{PROFILES[profile]['profile_role']}"] = account_profile
+        account_profiles[f"{short_name}-{account_name}-{PROFILES[type]['profile_role']}"] = account_profile
 
     logger.info("Backing up account profiles file.")
     _backup_file("config")
@@ -540,94 +517,148 @@ def configure_accounts_profiles(profile_name, region, organization_accounts, pro
         configure_profile(profile_identifier, profile_values)
 
 
+def mutually_exclusive(context, param, value):
+    """ Callback for command options --overwrite-existing-credentials and --skip-access-keys-setup mutual exclusivity verification. """
+    me_option = {
+        "overwrite_existing_credentials": "skip_access_keys_setup",
+        "skip_access_keys_setup": "overwrite_existing_credentials"
+    }
+
+    if value and context.params.get(me_option[param.name], False):
+        raise click.BadOptionUsage(option_name=param,
+                                   message=(f"Option {param.opts[0]} is mutually exclusive"
+                                            f" with option --{me_option[param.name].replace('_', '-')}."),
+                                   ctx=context)
+
+    return value
+
+
 @credentials.command()
-@click.option("--profile",
-              type=click.Choice(["bootstrap",
-                                 "management",
-                                 "security"],
+@click.option("--type",
+              type=click.Choice(["BOOTSTRAP",
+                                 "MANAGEMENT",
+                                 "SECURITY"],
                                 case_sensitive=False),
-              help="Profile credentials to set.")
-@click.option("--file",
+              required=True,
+              help="Type of credentials to set.")
+@click.option("--credentials-file",
               type=click.Path(exists=True, path_type=Path),
               help="Path to AWS cli credentials file.")
-@click.option("--only-accounts-profiles",
+@click.option("--overwrite-existing-credentials",
               is_flag=True,
-              help="Only update accounts' profiles, don't change key/secret.")
-@pass_state
-def update(state, profile, file, only_accounts_profiles):
-    """ Update credentials for the given profile.
+              callback=mutually_exclusive,
+              help=("Overwrite existing credentials if already configured.\n"
+                    "Mutually exclusive with --skip-access-keys-setup."))
+@click.option("--skip-access-keys-setup",
+              is_flag=True,
+              callback=mutually_exclusive,
+              help=("Skip access keys configuration. Continue on with assumable roles setup.\n"
+                   "Mutually exclusive with --overwrite-existing-credentials."))
+@click.option("--skip-assumable-roles-setup",
+              is_flag=True,
+              help="Don't configure the accounts assumable roles.")
+# TODO: Add --override-role-name parameter for non-default roles in accounts
+def configure(type, credentials_file, overwrite_existing_credentials, skip_access_keys_setup, skip_assumable_roles_setup):
+    """ Configure credentials for the project.
 
-    Only to be run after having initialized the project credentials. Generate the profiles for all
-    accounts in the project attaching MFA serial if the credentials require so.
-
-    Backup previously existent credentials if necessary and update the project configuration file with
-    the ids for all accounts.
+    It can handle the credentials required for the initial deployment of the project (BOOTSTRAP),
+    a management user (MANAGEMENT) or a devops/secops user (SECURITY).
     """
-    project_config = state.project_config
-
-    short_name = project_config.get("short_name") or _ask_for_short_name()
-    region = project_config.get("primary_region") or _ask_for_region()
-
-    if not _profile_is_configured(profile=f"{short_name}-bootstrap"):
-        logger.error("Credentials haven't been created yet, please run the credentials creation command first.")
+    if skip_access_keys_setup and skip_assumable_roles_setup:
+        logger.info("Nothing to do. Exiting.")
         return
 
-    profile = profile or _ask_for_profile()
-    profile_name = f"{short_name}-{profile}"
+    # Without the project templates no file can be rendered
+    if not (TEMPLATES_REPO_DIR / ".git").exists():
+        logger.info("Please initialize the Leverage project before configuring the credentials.")
+        raise Exit(1)
 
-    credentials_dir = AWSCLI_CONFIG_DIR / short_name
-    credentials_config = credentials_dir / "credentials"
-    profiles_config = credentials_dir / "config"
+    config_values = _load_configs_for_credentials()
 
-    already_configured = _profile_is_configured(profile=profile_name)
+    # Environment configuration variables are needed for the Leverage docker container
+    render_file(file=ENV_CONFIG_FILE, config=config_values)
 
-    if only_accounts_profiles:
-        if not already_configured:
-            logger.error("Credentials for this profile haven't been configured yet.\n"
-                         "Please re-run the command without the [bold]--only-account-profiles[/bold] flag.")
+    type = type.lower()
+    short_name = config_values.get("short_name")
+    profile = f"{short_name}-{type}"
+
+    already_configured = _profile_is_configured(profile=profile)
+    if already_configured and not (skip_access_keys_setup or overwrite_existing_credentials):
+        title_extra = "" if skip_assumable_roles_setup else " Continue on with assumable roles setup."
+
+        overwrite_existing_credentials = _ask_for_credentials_overwrite(
+            profile=profile,
+            skip_option_title=f"Skip credentials configuration.{title_extra}",
+            overwrite_option_title="Overwrite current credentials. Backups will be made."
+        )
+
+    do_configure_credentials = False if skip_access_keys_setup else overwrite_existing_credentials or not already_configured
+
+    if do_configure_credentials:
+        logger.info(f"Configuring [bold]{type}[/bold] credentials.")
+        configure_credentials(profile, credentials_file, make_backup=already_configured)
+        logger.info(f"[bold]{type.capitalize()} credentials configured in:[/bold]"
+                    f" {(AWSCLI_CONFIG_DIR / short_name / 'credentials').as_posix()}")
+
+        if not _credentials_are_valid(profile):
+            logger.error(f"Invalid {profile} credentials. Please check the given keys.")
             return
 
-    else:
-        if (not already_configured
-                or _ask_for_credentials_overwrite(
-                    profile=profile,
-                    skip_option_title="Skip credentials setup. Only update accounts' profiles.",
-                    overwrite_option_title="Overwrite current credentials. Backups will be made."
-                )
-            ):
-            logger.info(f"Configuring [bold]{profile}[/bold] credentials.")
-            configure_credentials(profile_name, file, make_backup=already_configured)
-            logger.info(f"[bold]{profile.capitalize()} credentials configured in:[/bold] {credentials_config.as_posix()}")
+    accounts = config_values.get("organization", {}).get("accounts", False)
+    # First time configuring bootstrap credentials
+    if type == "bootstrap" and not already_configured:
+        management_account = next((account for account in accounts if account["name"] == "management"), None)
 
-            if not _credentials_are_valid(profile=profile_name):
-                logger.error(f"Invalid {profile} credentials. Please check the given keys.")
-                return
+        if management_account:
+            logger.info("Fetching management account id.")
+            management_account_id =_get_management_account_id(profile=profile)
+            management_account["id"] = management_account_id
 
-    project_accounts = project_config.get("organization").get("accounts")
-    if _organization_is_created(profile=profile_name) and project_accounts:
-        logger.info("Configuring accounts' profiles.")
-        project_name = project_config.get("project_name")
+            project_config_file = load_project_config()
+            if project_config_file and "accounts" in project_config_file.get("organization", {}):
+                project_config_file["organization"]["accounts"] = accounts
 
-        logger.info("Fetching organization accounts.")
-        organization_accounts = _get_organization_accounts(profile=profile_name, project_name=project_name)
+                logger.info("Updating project configuration file.")
+                YAML().dump(data=project_config_file, stream=PROJECT_CONFIG)
 
-        configure_accounts_profiles(profile_name, region, organization_accounts, project_accounts)
-        logger.info(f"[bold]Account profiles configured in:[/bold] {profiles_config.as_posix()}")
+        skip_assumable_roles_setup = True
 
-        for account in project_accounts:
-            try: # Account in config file may not be already created
-                account["id"] = organization_accounts[account["name"]]
-            except KeyError:
-                continue
+    profile_for_organization = profile
+    # Security credentials don't have permission to access organization information
+    if type == "security":
+        for type_with_permission in ("management", "bootstrap"):
+            profile_to_check = f"{short_name}-{type_with_permission}"
 
-        logger.info("Updating project configuration file.")
-        YAML().dump(data=project_config, stream=PROJECT_CONFIG)
+            if _profile_is_configured(profile_to_check):
+                profile_for_organization = profile_to_check
+                break
 
-        # Update common.tfvars
-        render_file("config/common.tfvars")
+    if skip_assumable_roles_setup:
+        logger.info("Skipping assumable roles configuration.")
 
     else:
-        logger.info("No organization has been created yet or no accounts were found in project configuration file."
-                    " Skipping accounts' profiles configuration.")
+        logger.info("Attempting to fetch organization accounts.")
+        organization_accounts = _get_organization_accounts(profile_for_organization,
+                                                           config_values.get("project_name"))
+        logger.debug(f"Organization Accounts fetched: {organization_accounts}")
 
-    logger.info(f"Finished updating [bold]{profile}[/bold] credentials.")
+        if organization_accounts or accounts:
+            logger.info("Configuring assumable roles.")
+
+            configure_accounts_profiles(profile, config_values["primary_region"], organization_accounts, accounts)
+            logger.info(f"[bold]Account profiles configured in:[/bold]"
+                        f" {(AWSCLI_CONFIG_DIR / short_name / 'config').as_posix()}")
+
+            for account in accounts:
+                try: # Some account may not already be created
+                    account["id"] = organization_accounts[account["name"]]
+                except KeyError:
+                    continue
+
+        else:
+            logger.info("No organization has been created yet or no accounts were configured.\n"
+                        "Skipping assumable roles configuration.")
+
+    if PROJECT_COMMON_TFVARS.exists():
+        logger.info("Updating project's Terraform common configuration.")
+        render_file("config/common.tfvars", config=config_values)
