@@ -1,13 +1,10 @@
 """
     Module for managing Leverage projects.
 """
-import re
 from pathlib import Path
 from shutil import copy2
 from shutil import copytree
 from shutil import ignore_patterns
-from subprocess import run
-from subprocess import PIPE
 
 import click
 from click.exceptions import Exit
@@ -19,15 +16,17 @@ from leverage import logger
 from leverage.logger import console
 from leverage.path import get_root_path
 from leverage.path import NotARepositoryError
+from leverage._utils import git
 
 from leverage.modules.terraform import run as tfrun
 
 # Leverage related base definitions
 LEVERAGE_DIR = Path.home() / ".leverage"
-TEMPLATE_DIR = LEVERAGE_DIR / "template"
+TEMPLATES_REPO_DIR = LEVERAGE_DIR / "templates"
+TEMPLATE_DIR = TEMPLATES_REPO_DIR / "template"
 PROJECT_CONFIG_FILE = "project.yaml"
 TEMPLATE_PATTERN = "*.template"
-CONFIG_FILE_TEMPLATE = TEMPLATE_DIR / "le-resources" / PROJECT_CONFIG_FILE
+CONFIG_FILE_TEMPLATE = TEMPLATES_REPO_DIR / "le-resources" / PROJECT_CONFIG_FILE
 LEVERAGE_TEMPLATE_REPO = "https://github.com/binbashar/le-tf-infra-aws-template.git"
 IGNORE_PATTERNS = ignore_patterns(TEMPLATE_PATTERN, ".gitkeep")
 
@@ -88,19 +87,24 @@ def init():
         logger.info("No [bold]Leverage[/bold] config directory found in user's home. Creating.")
         LEVERAGE_DIR.mkdir()
 
-    # Leverage project template
-    if not TEMPLATE_DIR.exists():
-        TEMPLATE_DIR.mkdir()
+    # Leverage project templates
+    if not TEMPLATES_REPO_DIR.exists():
+        TEMPLATES_REPO_DIR.mkdir(parents=True)
 
-    if not (TEMPLATE_DIR / ".git").exists():
+    if not (TEMPLATES_REPO_DIR / ".git").exists():
         logger.info("No project template found. Cloning template.")
-        run(["git", "clone", LEVERAGE_TEMPLATE_REPO, TEMPLATE_DIR.as_posix()],
-            stdout=PIPE, stderr=PIPE, check=True)
+        git(f"clone {LEVERAGE_TEMPLATE_REPO} {TEMPLATES_REPO_DIR.as_posix()}")
         logger.info("Finished cloning template.")
 
+    else:
+        logger.info("Project template found. Updating.")
+        git(f"-C {TEMPLATES_REPO_DIR.as_posix()} checkout master")
+        git(f"-C {TEMPLATES_REPO_DIR.as_posix()} pull")
+        logger.info("Finished updating template.")
+
     # Leverage projects are git repositories too
     logger.info("Initializing git repository in project directory.")
-    run(["git", "init"], stdout=PIPE, stderr=PIPE, check=True)
+    git("init")
 
     # Project configuration file
     if not PROJECT_CONFIG.exists():
@@ -146,10 +150,12 @@ def _copy_project_template(config):
     Args:
         config (dict): Project configuration.
     """
-    # TODO: Set the project template version (checkout the corresponding tag) based on the
-    # project configuration file (under meta.version)
     logger.info("Creating project directory structure.")
 
+    # Copy .gitignore file
+    copy2(src=TEMPLATE_DIR / ".gitignore",
+          dst=PROJECT_ROOT / ".gitignore")
+
     # Root config directory
     copytree(src=TEMPLATE_DIR / CONFIG_DIRECTORY,
              dst=PROJECT_ROOT / CONFIG_DIRECTORY,
@@ -177,7 +183,7 @@ def value(dictionary, key):
 
 
 # Jinja environment used for rendering the templates
-JINJA_ENV = Environment(loader=FileSystemLoader(TEMPLATE_DIR.as_posix()),
+JINJA_ENV = Environment(loader=FileSystemLoader(TEMPLATES_REPO_DIR.as_posix()),
                         trim_blocks=False,
                         lstrip_blocks=False)
 JINJA_ENV.filters["value"] = value
@@ -194,21 +200,21 @@ def _render_templates(template_files, config, source=TEMPLATE_DIR, destination=P
         destination (Path, optional): Destination where to render the templates. Defaults to PROJECT_ROOT.
     """
     for template_file in template_files:
-        logger.debug(f"Template: {template_file.as_posix()}")
-        template_location = template_file.relative_to(source)
+        template_location = template_file.relative_to(TEMPLATES_REPO_DIR)
 
         template = JINJA_ENV.get_template(template_location.as_posix())
         rendered_template = template.render(config)
 
-        if (template_location.parent.name == ""
-                or template_location.parent.name == CONFIG_DIRECTORY
-                or template_location.parent.parent.name == "global"):
-            rendered_location = destination / template_location
+        rendered_location = template_file.relative_to(source)
+        if (rendered_location.parent.name == ""
+                or rendered_location.parent.name == CONFIG_DIRECTORY
+                or rendered_location.parent.parent.name == "global"):
+            rendered_location = destination / rendered_location
 
         else:
             region_name = template_location.parent.parent.name
-            template_location = template_location.as_posix().replace(region_name, config[region_name])
-            rendered_location = destination / Path(template_location)
+            rendered_location = rendered_location.as_posix().replace(region_name, config[region_name])
+            rendered_location = destination / Path(rendered_location)
 
         rendered_location = rendered_location.with_suffix("")
 
@@ -264,6 +270,7 @@ def load_project_config():
         dict:  Project configuration.
     """
     if not PROJECT_CONFIG.exists():
+        logger.debug("No project config file found.")
         return {}
 
     try:
@@ -303,19 +310,21 @@ def create():
     logger.info("Finished setting up project.")
 
 
-def render_file(file):
+def render_file(file, config=None):
     """ Utility to re-render specific files.
 
     Args:
         file (str): Relative path to file to render.
+        config (dict, optional): Config used to render file.
 
     Returns:
         bool: Whether the action succeeded or not
     """
-    # TODO: Make use of internal state
-    config = load_project_config()
     if not config:
-        return False
+        # TODO: Make use of internal state
+        config = load_project_config()
+        if not config:
+            return False
 
     try:
         _render_templates([TEMPLATE_DIR / f"{file}.template"], config=config)
