@@ -28,6 +28,7 @@ from leverage.path import NotARepositoryError
 
 # Terraform image definitions
 TERRAFORM_IMAGE = "binbash/terraform-awscli-slim"
+DEFAULT_IMAGE_TAG = "1.0.9"
 TERRAFORM_BINARY = "/bin/terraform"
 TERRAFORM_MFA_ENTRYPOINT = "/root/scripts/aws-mfa/aws-mfa-entrypoint.sh"
 WORKING_DIR = "/go/src/project"
@@ -109,11 +110,11 @@ def ensure_image(docker_client, image, tag):
         logger.info(info)
 
 
-def run(entrypoint=None, command="", args=None, enable_mfa=True, interactive=True):
+def run(entrypoint=TERRAFORM_BINARY, command="", args=None, enable_mfa=True, interactive=True):
     """ Run a command on a Leverage docker container.
 
     Args:
-        entrypoint (str, optional): Entrypoint to use in the container, overrides the one defined in the image. Defaults to None.
+        entrypoint (str, optional): Entrypoint to use in the container, overrides the one defined in the image. Defaults to `/bin/terraform`.
         command (str, optional): Command to run. Defaults to "".
         args (list(str)), optional): Command arguments. Defaults to None.
         enable_mfa (bool, optional): Whether to enable multi factor authentication. Defaults to True.
@@ -143,17 +144,16 @@ def run(entrypoint=None, command="", args=None, enable_mfa=True, interactive=Tru
     if not aws_credentials_directory.exists():
         aws_credentials_directory.mkdir(parents=True)
 
-    terraform_image_tag = env.get("TERRAFORM_IMAGE_TAG", "latest")
+    terraform_image_tag = env.get("TERRAFORM_IMAGE_TAG", DEFAULT_IMAGE_TAG)
     ensure_image(docker_client=docker_client,
                  image=TERRAFORM_IMAGE,
                  tag=terraform_image_tag)
 
-    entrypoint = TERRAFORM_BINARY if entrypoint is None else entrypoint
-
     mounts = [
         Mount(target=WORKING_DIR, source=CWD, type="bind"),
-        Mount(target="/root/.ssh", source=f"{HOME}/.ssh", type="bind"),
-        Mount(target="/etc/gitconfig", source=f"{HOME}/.gitconfig", type="bind")
+        # Mount(target="/root/.ssh", source=f"{HOME}/.ssh", type="bind"), # SSH keys for Ansible
+        # Mount(target="/etc/gitconfig", source=f"{HOME}/.gitconfig", type="bind"), # Git user configuration
+        Mount(target=f"/root/tmp/{project}", source=f"{HOME}/.aws/{project}", type="bind")
     ]
     if Path(str(CONFIG)).exists() and Path(str(ACCOUNT_CONFIG)).exists():
         mounts.extend([
@@ -163,30 +163,33 @@ def run(entrypoint=None, command="", args=None, enable_mfa=True, interactive=Tru
 
     environment = {
         "AWS_SHARED_CREDENTIALS_FILE": f"/root/.aws/{project}/credentials",
-        "AWS_CONFIG_FILE": f"/root/.aws/{project}/config"
+        "AWS_CONFIG_FILE": f"/root/.aws/{project}/config",
+        "BACKEND_CONFIG_FILE": BACKEND_TFVARS,
+        "COMMON_CONFIG_FILE": COMMON_TFVARS,
+        "SRC_AWS_CONFIG_FILE": f"/root/tmp/{project}/config",
+        "SRC_AWS_SHARED_CREDENTIALS_FILE": f"/root/tmp/{project}/credentials",
+        "AWS_CACHE_DIR": f"/root/tmp/{project}/cache",
+        "MFA_SCRIPT_LOG_LEVEL": get_mfa_script_log_level()
     }
 
-    enable_mfa = enable_mfa and env.get("MFA_ENABLED") == "true"
+    if entrypoint == TERRAFORM_BINARY:
+        enable_mfa = enable_mfa and env.get("MFA_ENABLED") == "true"
+
     if enable_mfa:
-        entrypoint = TERRAFORM_MFA_ENTRYPOINT
-        if command:
-            entrypoint = f"{TERRAFORM_MFA_ENTRYPOINT} -- {TERRAFORM_BINARY}"
+        if Path(CWD).parents[1] != Path(ACCOUNT):
+            logger.error("This command can only run at [bold]layer[/bold] level.")
+            raise Exit(1)
 
-        mounts.extend([
-            Mount(target=f"/root/tmp/{project}", source=f"{HOME}/.aws/{project}", type="bind")
-        ])
-
-        environment.update({
-            "BACKEND_CONFIG_FILE": BACKEND_TFVARS,
-            "COMMON_CONFIG_FILE": COMMON_TFVARS,
-            "SRC_AWS_CONFIG_FILE": f"/root/tmp/{project}/config",
-            "SRC_AWS_SHARED_CREDENTIALS_FILE": f"/root/tmp/{project}/credentials",
-            "AWS_CACHE_DIR": f"/root/tmp/{project}/cache",
-            "MFA_SCRIPT_LOG_LEVEL": get_mfa_script_log_level()
-        })
+        if command or entrypoint != TERRAFORM_BINARY:
+            entrypoint = f"{TERRAFORM_MFA_ENTRYPOINT} -- {entrypoint}"
+        else:
+            entrypoint = TERRAFORM_MFA_ENTRYPOINT
 
     else:
-        mounts.append(Mount(target=f"/root/.aws/{project}", source=f"{HOME}/.aws/{project}", type="bind"))
+        environment.update({
+            "AWS_CONFIG_FILE": f"/root/tmp/{project}/config",
+            "AWS_SHARED_CREDENTIALS_FILE": f"/root/tmp/{project}/credentials"
+        })
 
     args = [] if args is None else args
     command = " ".join([command] + args)
@@ -216,6 +219,7 @@ def run(entrypoint=None, command="", args=None, enable_mfa=True, interactive=Tru
         if interactive:
             dockerpty.start(client=docker_client.api,
                             container=container)
+            container_exit_code = docker_client.api.inspect_container(container)["State"]["ExitCode"]
         else:
             docker_client.api.start(container)
             container_exit_code = docker_client.api.wait(container)["StatusCode"]
@@ -257,7 +261,10 @@ CONTEXT_SETTINGS = {"ignore_unknown_options": True}
 def init(no_backend, args):
     """ Initialize this layer. """
     backend_config = ["-backend=false" if no_backend else f"-backend-config={BACKEND_TFVARS}"]
-    run(command="init", args=backend_config + list(args))
+    exit_code, _ = run(command="init", args=backend_config + list(args))
+
+    if exit_code:
+        raise Exit(exit_code)
 
 
 @terraform.command(context_settings=CONTEXT_SETTINGS)
@@ -265,7 +272,10 @@ def init(no_backend, args):
 @check_directory
 def plan(args):
     """ Generate an execution plan for this layer. """
-    run(command="plan", args=TF_DEFAULT_ARGS + list(args))
+    exit_code, _ = run(command="plan", args=TF_DEFAULT_ARGS + list(args))
+
+    if exit_code:
+        raise Exit(exit_code)
 
 
 @terraform.command(context_settings=CONTEXT_SETTINGS)
@@ -273,7 +283,10 @@ def plan(args):
 @check_directory
 def apply(args):
     """ Build or change the infrastructure in this layer. """
-    run(command="apply", args=TF_DEFAULT_ARGS + list(args))
+    exit_code, _ = run(command="apply", args=TF_DEFAULT_ARGS + list(args))
+
+    if exit_code:
+        raise Exit(exit_code)
 
 
 @terraform.command(context_settings=CONTEXT_SETTINGS)
@@ -281,7 +294,7 @@ def apply(args):
 @check_directory
 def output(args):
     """ Show all output variables of this layer. """
-    run(command="output", args=list(args), enable_mfa=False)
+    run(command="output", args=list(args))
 
 
 @terraform.command(context_settings=CONTEXT_SETTINGS)
@@ -299,9 +312,13 @@ def version():
 
 
 @terraform.command()
-def shell():
+@click.option("--mfa",
+              is_flag=True,
+              default=False,
+              help="Enable Multi Factor Authentication upon launching shell.")
+def shell(mfa):
     """ Open a shell into the Terraform container in this layer. """
-    run(entrypoint="/bin/sh", enable_mfa=False)
+    run(entrypoint="/bin/sh", enable_mfa=mfa)
 
 
 @terraform.command("format")
@@ -328,15 +345,14 @@ def validate():
 @check_directory
 def _import(address, _id):
     """ Import a resource. """
-    run(command="import", args=TF_DEFAULT_ARGS + [address, _id])
+    exit_code, _ = run(command="import", args=TF_DEFAULT_ARGS + [address, _id])
+
+    if exit_code:
+        raise Exit(exit_code)
 
 
 @terraform.command(context_settings=CONTEXT_SETTINGS)
-@click.option("--no-mfa",
-              is_flag=True,
-              default=False,
-              help="Disable MFA.")
 @click.argument("args", nargs=-1)
-def aws(no_mfa, args):
+def aws(args):
     """ Run a command in AWS cli. """
-    run(entrypoint="/usr/bin/aws", args=list(args), enable_mfa=not no_mfa)
+    run(entrypoint="/usr/bin/aws", args=list(args), enable_mfa=False)
